"""Package initialization."""

__version__ = "2022.6.0"

__title__ = "edgetest-hub"
__description__ = "Edgetest hub plugin"
__url__ = "https://github.com/capitalone/edgetest-hub"
__uri__ = __url__
__doc__ = __description__ + " <" + __uri__ + ">"

__author__ = "Faisal Dosani"
__email__ = "faisal.dosani@capitalone.com"
